"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JUnitReporter = void 0;
const utils_1 = require("../utils");
// cli currently has spaces in multiples of four for junit format
const tab = '    ';
const timeProperties = [
    'testExecutionTimeInMs',
    'testTotalTimeInMs',
    'commandTimeInMs'
];
// properties not in cli junit spec
const skippedProperties = ['skipRate', 'totalLines', 'linesCovered'];
class JUnitReporter {
    format(testResult) {
        const { summary, tests } = testResult;
        let output = `<?xml version="1.0" encoding="UTF-8"?>\n`;
        output += `<testsuites>\n`;
        output += `${tab}<testsuite name="force.apex" `;
        output += `timestamp="${new Date(summary.testStartTime).toISOString()}" `;
        output += `hostname="${summary.hostname}" `;
        output += `tests="${summary.testsRan}" `;
        output += `failures="${summary.failing}"  `;
        output += `errors="0"  `;
        output += `time="${utils_1.msToSecond(summary.testExecutionTimeInMs)}">\n`;
        output += this.buildProperties(testResult);
        output += this.buildTestCases(tests);
        output += `${tab}</testsuite>\n`;
        output += `</testsuites>\n`;
        return output;
    }
    buildProperties(testResult) {
        let junitProperties = `${tab}${tab}<properties>\n`;
        Object.entries(testResult.summary).forEach(([key, value]) => {
            if (this.isEmpty(value) || skippedProperties.includes(key)) {
                return;
            }
            if (timeProperties.includes(key)) {
                value = `${utils_1.msToSecond(value)} s`;
                key = key.replace('InMs', '');
            }
            if (key === 'outcome' && value === 'Passed') {
                value = 'Successful';
            }
            junitProperties += `${tab}${tab}${tab}<property name="${key}" value="${value}"/>\n`;
        });
        junitProperties += `${tab}${tab}</properties>\n`;
        return junitProperties;
    }
    buildTestCases(tests) {
        let junitTests = '';
        for (const testCase of tests) {
            junitTests += `${tab}${tab}<testcase name="${testCase.methodName}" classname="${testCase.apexClass.fullName}" time="${utils_1.msToSecond(testCase.runTime)}">\n`;
            if (testCase.outcome === "Fail" /* Fail */ ||
                testCase.outcome === "CompileFail" /* CompileFail */) {
                junitTests += `${tab}${tab}${tab}<failure message="${testCase.message}">`;
                if (testCase.stackTrace) {
                    junitTests += `<![CDATA[${testCase.stackTrace}]]>`;
                }
                junitTests += `</failure>\n`;
            }
            junitTests += `${tab}${tab}</testcase>\n`;
        }
        return junitTests;
    }
    isEmpty(value) {
        if (value === null ||
            value === undefined ||
            (typeof value === 'string' && value.length === 0)) {
            return true;
        }
        return false;
    }
}
exports.JUnitReporter = JUnitReporter;
//# sourceMappingURL=junitReporter.js.map